with open('ftp_server.txt','r',encoding='utf-8') as f:
    COMPLETE_SERVER = f.readline()[:-1].split(':')
    ENCODING = f.readline()[:-1]
HOST,PORT = COMPLETE_SERVER
PORT = int(PORT)
if PORT == 21:
    SERVER = HOST
else:
    SERVER = COMPLETE_SERVER



from win32api import MessageBox
import win32con
from os import _exit


warn = lambda m: MessageBox(0, str(m), "Warning",\
                            win32con.MB_ICONEXCLAMATION|win32con.MB_TOPMOST)


class Warner:

    def __init__(self,file,message):
        self.message = message
        self.file = file
    def write(self,message):
        self.message += message
        return
    def __del__(self):
        from time import ctime

        warn(self.message)
        with open(self.file,'a') as f:
            f.write('\n'+'-'*20+ctime()+'-'*20+'\n')
            f.write(self.message)
        print(self.message)

def log_and_exit(file = 'log.txt', message = '', to_exit = True):
    from traceback import print_exc

    print_exc(file=Warner(file, message))
    if to_exit == True:
        _exit(1)

def log_it(file = 'log.txt', to_exit = True):
    def dec(f):
        def func(*args, **kargs):
            try:
                f(*args, **kargs)
            except:
                log_and_exit(file = file, to_exit = to_exit)
        return func
    return dec


##@log_it
##def h():
##    return 0/0
##h()



import sys
from win32gui import FindWindowEx, GetWindowText
from os import popen,makedirs,stat,_exit
from os.path import isfile
from re import match
from . import FTPDownloader
from . import messager
import yaml

with open('download_config.yaml','r',encoding='utf-8') as f:
    rules,setting=yaml.load_all(f)
LOCAL_PREFIX=setting['LOCAL_PREFIX']


##notify=lambda m: MessageBox(0, str(m), "Warning", 48)
def get_explorer_path():
    hwnd = 0
    children = ('CabinetWClass','WorkerW','ReBarWindow32','Address Band Root',
                'msctls_progress32','Breadcrumb Parent','ToolbarWindow32')
    for child_class in children:
        hwnd = FindWindowEx(hwnd,0,child_class,None)
    return GetWindowText(hwnd)

def get_local_path(ftp_path,file):
    try:
        p1=r'地址: ftp://(.*):(.*)\@6\.163\.193\.243(.*)'
        ftp_info=match(p1,ftp_path).groups()
    except:
        p1 = r'地址: ftp://(.*)\@192\.168\.123\.99\:2121(.*)'
        ftp_info=match(p1,ftp_path).groups()
        ftp_info=(ftp_info[0],'123',ftp_info[1])
##    if ftp_info[2]=='':
##        ftp_info=ftp_info[0:2]+('.',)
    s_path=ftp_info[0]+ftp_info[2]
    ftp_info=ftp_info[0:2]+(parse_CH(ftp_info[2]),)
    p2=r'.*\\(.*)\[.*\](.*)'
    print(p2,file)
    file_name=''.join(match(p2,file).groups())
    file_name=file_name.replace('_',' ')
    ftp_info+=(file_name,)
    #notify(str(ftp_info))
    local_path=LOCAL_PREFIX
    for k, v in rules.items():
        result=match(k,s_path)
        if not result is None:
            try:
                match_path=parse_CH(result.group(1))
                local_path+=v+'\\'+match_path
            except Exception:
                local_path+=v
    local_path=local_path.replace('/','\\')
    print(local_path,type(local_path))
    makedirs(local_path,exist_ok=True)
    return local_path+file_name,ftp_info

def parse_CH(s):
    l=len(s)
    i=0
    r=''
    while i<l:
        if s[i] == '%':
            hexnum=s[i+1:i+3]+s[i+4:i+6]
            print(hexnum)
            r+=bytes.fromhex(hexnum).decode('gbk')
            i+=6
        else:
            r+=s[i]
            i+=1
    return r

@messager.log_it(file='log_handler.txt')
def main(file=None):
    if file is None:
        file=sys.argv[1]

    dest,ftp_info=get_local_path(get_explorer_path(),file)
    FTPDownloader.init(*ftp_info[0:2])
    FTPDownloader.download(*ftp_info[2:],dest=dest)

if __name__=='__main__':
    main()



from sys import argv
from os import system, popen, chdir
import os.path
from . import messager

@messager.log_it(file = 'log_main.txt')
def main():
    cmd = argv[1]
    if cmd == 'gui':
        opt = argv[2] if len(argv) == 3 else ''
        if opt == '':
            from .NewFTP import main
            main()
        if opt == 'config':
            system('explorer gui_config.yaml')
        if opt == 'style.show':
            system('explorer Styles')
    ##    if opt == 'style.install':
    ##        import shutil
    ##        file = argv[3]
    ##        if os.path.isfile(file):
    ##            filename = os.path.split(file)
    ##            shutil.copy2(file,os.path.expanduser('~/.NewFTP/Styles/'))
    ##        if os.isdir(file):

        if opt == 'log.show':
            system('explorer gui_log.txt')
main()



from ftplib import FTP,error_perm
from sys import stdout
import time
from os import _exit,stat,utime,popen,system
from os.path import isfile,dirname,abspath,splitext
from tqdm import tqdm
try:
    from . import messager
    from .ftp_parser import HOST, PORT, ENCODING
except ImportError:
    import messager
    from ftp_parser import HOST, PORT, ENCODING

ftp = FTP()

ftp.encoding = ENCODING
USER=''
PASSWORD=''
SILENT=1024*800
PRINTING=False
ASK_FILE = '''检测到本地文件{0}，
是否覆盖？

Local file already exist.
Copy any way?
'''

class FileTracker():
    def __init__(self,filename,filesize=None):
        self.file=open(filename,'wb')
        self.open_it=None
        if splitext(filename)[1] in ('.mp4','.mp3'):
            popen('explorer "%s"'%filename)
            self.open_it=False
        self.filesize=filesize
        self.pbar=tqdm(total=filesize,unit='B',unit_scale=True,ncols=60,
            bar_format='{l_bar}{n_fmt}/{total_fmt}|{rate_fmt}{bar}{remaining}')
    def write(self,buff):
        self.pbar.update(len(buff))
        self.file.write(buff)
    def  close(self):
        self.file.close()
        return self.open_it

def init(user,password):
    global USER,PASSWORD
    USER,PASSWORD=user,password
    ftp.connect(HOST,PORT)
    ftp.login(user,password)

def file_conflict(time,size):
    from win32api import MessageBox
    import win32con

    def compare(a,b):
        if a == b:return 0
        elif a > b:return 1
        else:return -1

    status = (compare(*time),compare(*size))
    if 1 in status:
        time_description = {1:'较新',-1:'较旧'}
        size_description = {1:'较大',-1:'较小'}
        description = time_description.get(status[0],'')+\
                      size_description.get(status[1],'')
        result = MessageBox(win32con.NULL,ASK_FILE.format(description),\
                            '文件冲突 (File Conflict)',\
                            win32con.MB_YESNO|win32con.MB_ICONQUESTION|\
                            win32con.MB_TOPMOST)
        # Yes:6, No:7
        return 7-result
    elif status == (0,0):
        return 0
    else:
        return 1

def just_download(directory,filename,dest,ftp_mtime,ftp_filesize):
    if PRINTING == True:
        system('mode 100,7')
        system('color f2')
        system('title A simple downloader')
        print('This is a simple FTP downloader')
        print()
        print('File to download:',directory+filename)
        print('Local file name:',dest)
        print()
        local_file=FileTracker(dest,ftp_filesize)
    else:
        local_file=open(dest,'wb')
    ftp.retrbinary('RETR %s'%filename,local_file.write)
    open_flag=local_file.close()
    utime(dest,(ftp_mtime,ftp_mtime))
    if open_flag is None:
        popen('explorer "%s"'%dest)

def download(directory,filename,dest):
    ftp.cwd(directory)
    try:
        L=ftp.sendcmd('MDTM %s'%filename)
    except error_perm:
        filename=filename.replace(' ','_')
        L=ftp.sendcmd('MDTM %s'%filename)
    dir_t=L[4:8]+'-'+L[8:10]+'-'+L[10:12]+' '+L[12:14]+':'+L[14:16]+':'+L[16:18]
    timeArray = time.strptime(dir_t, "%Y-%m-%d %H:%M:%S")
    ftp_mtime = int(time.mktime(timeArray))
    ftp_filesize=ftp.size(filename)
    if isfile(dest):
        local_file=stat(dest)
        # messager.warn((local_file.st_mtime,ftp_mtime),(local_file.st_size,ftp_filesize))
        result = file_conflict((local_file.st_mtime,ftp_mtime),
                               (local_file.st_size,ftp_filesize))
        if result == 0:
            popen('explorer "%s"'%dest)
            ftp.close()
            _exit(0)
        # if ftp_mtime == local_file.st_mtime:
        # if ftp_mtime >= local_file.st_mtime:
        #     popen('DEL "%s"'%dest)

    if ftp_filesize < SILENT:
        just_download(directory,filename,dest,ftp_mtime,ftp_filesize)
    else:
        downloader = dirname(abspath(__file__))+'\\FTPDownloader.py'
        cmd='python %s %s %s "%s" "%s" "%s" %d %d'\
             %(downloader,USER,PASSWORD,directory,filename,dest,ftp_mtime,ftp_filesize)
        # if you use -m NewFTP.FTPDownloader, python can't recognize the package
        #cmd='python -m NewFTP.FTPDownloader %s %s "%s" "%s" "%s" %d %d & pause'\
        #     %(USER,PASSWORD,directory,filename,dest,ftp_mtime,ftp_filesize)
        #messager.warn(cmd)
        system(cmd)
    ftp.close()


@messager.log_it(file = 'log_download.txt')
def main(user,password,directory,filename,dest,ftp_mtime,ftp_filesize):
    global PRINTING
    from sys import argv

    PRINTING=True

    init(user,password)
    ftp.cwd(directory)
    just_download(directory,filename,dest,int(ftp_mtime),int(ftp_filesize))
    ftp.close()
    time.sleep(0.5)
##messager.warn(main)
if __name__ == '__main__':
    from sys import argv
    main(*argv[1:8])



import win32con
from win32gui import FindWindow, PostMessage, SetForegroundWindow
from os import _exit
def find():
    hwnd = FindWindow(None,'oh-my-ftp')
    if hwnd:
        PostMessage(hwnd, win32con.WM_LBUTTONDOWN, 0,(1<<16)+1)
        PostMessage(hwnd, win32con.WM_LBUTTONUP, 0,(1<<16)+1)
        SetForegroundWindow(hwnd)
        _exit(0)
    return
find()



'''In this module , a positive x means it goes right,
and a positive y means it goes down.'''
from pygame.locals import *
class Direction:
     def __init__(self,x,y):
          self.x=x
          self.y=y
          self.direc=(x,y)
     def __str__(self):
          return ('x:%s y:%s'%(self.x,self.y))
     def __repr__(self):
          return ('x:%s y:%s'%(self.x,self.y))
class Pattern:
     def __init__(self,up,down,left,right):
          self.up=up
          self.down=down
          self.right=right
          self.left=left
          self._tuple=(up,down,left,right)
     def get_direction(self,key):
          if key in self._tuple:
               num=self._tuple.index(key)
               if num<=1:
                    return Direction(0,num*2-1)
               else:
                    return Direction(num*2-5,0)
patterns={
     'UDLR':Pattern(273,274,276,275),
     'WSAD':Pattern(119,115,97,100),}

def get_key_direction(key_event):
     #direction_dict={}
     for key,pattern in patterns.items():
          tp=pattern.get_direction(key_event.key)
          if tp:
               return key,tp
     #return direction_dict
def add_pattern(string):
     assert len(string)==4,'Must four'
     #print(Pattern(*(list(map(lambda x:ord(x),string.lower())))))
     patterns.update({string:Pattern(*(list(map(lambda x:ord(x),string.lower()))))})
def remove_pattern(string):
     patterns.pop(string,None)
def get_mouse_direction(start_pos,end_pos)          :
     relx,rely=end_pos[0]-start_pos[0],end_pos[1]-start_pos[1]
     if relx**2+rely**2>20:
          if abs(relx)>2*abs(rely):
               gox=int(relx/abs(relx))
               goy=0
          elif abs(rely)>2*abs(relx):
               goy=int(rely/abs(rely))
               gox=0
          else:return 0,0
          return gox,goy
     return 0,0



from os import chdir
from os.path import expanduser
try:
    chdir(expanduser('~/.NewFTP'))
except FileNotFoundError:
    from shutil import copytree
    from os.path import dirname, abspath

    src = dirname(abspath(__file__))+'\\data'
    copytree(src, expanduser('~/.NewFTP'))
    chdir(expanduser('~/.NewFTP'))



import setuptools
from setuptools.command.install import install

def short_target(filename,dest,arg):
    import os, sys
    import pythoncom
    from win32com.shell import shell, shellcon


    desktop_path = shell.SHGetFolderPath (0, shellcon.CSIDL_DESKTOP, 0, 0)
    shortcut_path = os.path.join (desktop_path, filename)
    if not os.path.exists(shortcut_path):
        shortcut = pythoncom.CoCreateInstance (
            shell.CLSID_ShellLink,
            None,
            pythoncom.CLSCTX_INPROC_SERVER,
            shell.IID_IShellLink
        )
        with open (shortcut_path,'w'):
            pass
        persist_file = shortcut.QueryInterface (pythoncom.IID_IPersistFile)
        persist_file.Load (shortcut_path)
        shortcut.SetPath(dest)
        shortcut.SetArguments(arg)
        shortcut.SetHotkey(114)
        persist_file.Save (shortcut_path, 0)

def register(command):
    import winreg
    key=winreg.OpenKey(winreg.HKEY_CLASSES_ROOT,
                           r'ftp\shell\open\command',
                           0,winreg.KEY_WRITE)
    winreg.SetValueEx(key, "", 0, winreg.REG_SZ, command)
    key=winreg.OpenKey(winreg.HKEY_CURRENT_USER,
                       r'Software\Microsoft\Windows\Shell\Associations\UrlAssociations\ftp\UserChoice',
                       0,winreg.KEY_WRITE)
    winreg.SetValueEx(key, "Progid", 0, winreg.REG_SZ, "")
    
class CustomInstall(install):
    def run(self):
        from shutil import copytree
        from os.path import dirname, abspath, expanduser, isdir, split
        from sys import executable
        install.run(self)
        if not isdir(expanduser('~/.NewFTP')):
            src = dirname(abspath(__file__))+'\\NewFTP\\data'
            copytree(src, expanduser('~/.NewFTP'))
        short_target("FTP.lnk",'"%s\\pythonw.exe"'%split(executable)[0],
                     '-m NewFTP.NewFTP')
        register('"{0}\\pythonw.exe" -m NewFTP.PyFTPHandler "\%1"'\
                 .format(split(executable)[0]))

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="NewFTP",
    version="0.0.1",
    author="AllanChain",
    author_email="txsmlf@gmail.com",
    description="A GUI program to login the FTP system and a background program to manage FTP file downloading",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/AllanChain/NewFTP",
    packages=setuptools.find_packages(),
    package_data={
        '':['*.pyw','data/*','data/Styles/*','data/Styles/Img/*']},
    install_requires=["PyYAML","python-box","pygame","tqdm","pywin32"],
    cmdclass={'install': CustomInstall},
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU Affero General Public License v3",
        "Operating System :: Microsoft :: Windows :: Windows 7",
        "Development Status :: 3 - Alpha",
    ],
)



from .finder import find
import pygame
from pygame.locals import *
from os import popen, environ, _exit
import os.path
from win32gui import FindWindow, SetWindowPos, PostMessage, GetCursorPos,\
     SetForegroundWindow, ShowWindow
#from win32api import MessageBox
from . import direction
import win32con
import yaml
from box import SBox as Box
from . import messager
from .ftp_parser import SERVER


class NameManager:
    def __init__(self,usr,pas_dict,perpage):
        self.acnts=tuple(usr.items())
        self.pas_dict=pas_dict
        self.page = 0
        self.perpage = perpage
        self.maxpage = len(usr)//perpage
        self.total=len(self.acnts)
    def get_acnt(self, i):
        num = self.page*self.perpage+i
        if not 0 <= i <= self.perpage-1:
            return ('','')
        return self.acnts[num] if num<self.total else ('','')
    def get_usr(self,i):
        return self.get_acnt(i)[1]
    def get_name(self, i):
        return self.get_acnt(i)[0]
    def pageturn(self,pg):
        self.page = max(0,min(self.maxpage, self.page+pg))
    def launch(self, num):
        name=self.get_usr(num)
        print(num)
        if name.startswith('$'):
            #Out command
            #e.g. $python -m some_module
            cmd = name[1:]
        elif '/' in name or '\\' in name:
            #Directory, open with explorer.exe
            #Or open file with default application
            #e.g. D:\Desctop\mess\
            #e.g. ~/PythonPro/
            #e.g. D:\Desctop\1.txt
            cmd = 'start explorer "%s"'%name
        elif name.isalpha() or name == '':
            #Ensure tha acnt is valid
            name1 = name+':'+self.pas_dict.get(name,'123')+'@' if name else ''
            #If name is '', no :@ needed
            #here get(name,'123') means default password is '123'
            print (name1)
            cmd = 'start explorer ftp://%s%s'%(name1,SERVER)
        popen(cmd)
def C(color):
    '''Color converting'''
    if isinstance(color, tuple):
        return color
    if isinstance(color, int):
        #In case that color is 580646 stuff
        color = str(color)
    if isinstance(color, str):
        if color.startswith('#'):
            color = color[1:]
        if len(color) == 6:
            n = int(color, base=16)
            return (n>>16) % 256, (n>>8) % 256, n % 256
def load():
    with open('gui_config.yaml','r',encoding='utf-8') as f:
        usr,pas,sty = yaml.load_all(f)
    try:
        with open('styles/' + sty['style'] + '.yaml',encoding='utf-8') as f:
    #Here store some attributes to access them conveniently
            new_style = Box(yaml.load(f))
        parents = [sty['style']]
        while 'parent' in new_style and not new_style['parent'] in parents:
            parents.append(new_style['parent'])
            with open('styles/%s.yaml'%new_style['parent'],encoding='utf-8') as f:
                older_style = Box(yaml.load(f))
                new_style = older_style.update(new_style)
                new_style = older_style
        if not 'Win7' in parents:
            with open('Styles/Win7.yaml', encoding='utf-8') as f:
                # Using Win7 as ultimate parent
                style = Box(yaml.load(f))
                style.update(new_style)
        else:
            style = new_style
    except FileNotFoundError:
        messager.warn( "样式文件不可用！将使用默认样式")
        with open('Styles/Win7.yaml', encoding='utf-8') as f:
            style = Box(yaml.load(f))
    style.maximum.width = style.maximum.cols*style.maximum.block.width
    style.maximum.height = style.maximum.rows*style.maximum.block.height
    mgr = NameManager(usr,pas,style.maximum.cols*style.maximum.rows-1)
    #print(usr,pas)
    return mgr,style

def draw_bg(style):
    BGSurf = pygame.surface.Surface((style.maximum.width,style.maximum.height))
    BGSurf.fill(C(style.maximum.color.background))
    for i in range(style.maximum.cols):
        for j in range(style.maximum.rows):
             pygame.draw.rect(BGSurf, C(style.maximum.color.border),
                             (i*style.maximum.block.width,
                              j*style.maximum.block.height,
                              style.maximum.block.width,
                              style.maximum.block.height),
                             style.maximum.border)
    if not style.maximum.image is None:
        BG = pygame.image.load('Styles/'+style.maximum.image.path)
        BG = pygame.transform.scale(BG,(style.maximum.width,style.maximum.height))
        BG.set_alpha(style.maximum.image.alpha)
        BGSurf.blit(BG,(0, 0))
    BGMSurf = pygame.surface.Surface((style.minimum.width,style.minimum.height))
    BGMSurf.fill(C(style.minimum.color.background))
    pygame.draw.rect(BGMSurf,C(style.minimum.color.border),(0, 0, style.minimum.width,
                    style.minimum.height), style.minimum.border)
    return BGSurf, BGMSurf

@messager.log_it(file = 'log_gui.txt')
def main():
    global MINI
    find()
    try:
        mgr,style = load()
    except Exception:
        messager.log_and_exit(message = "配置文件加载失败！")
    environ['SDL_VIDEO_WINDOW_POS']='%d,%d'%(style.maximum.pos.x,style.maximum.pos.y)
    DIS = pygame.display.set_mode((style.maximum.width,style.maximum.height), NOFRAME)
    MINI = False
    MOVING = False
    find()
    pygame.display.set_caption('oh-my-ftp')
    hwnd = FindWindow(None,'oh-my-ftp')
    BGSurf,BGMSurf = draw_bg(style)
    mini_clock = pygame.time.Clock()
    mini_clock.tick()
    pygame.font.init()
    FontObj = pygame.font.SysFont(style.font.font, style.font.size)
    def draw_text():
        if MINI == True:
            DIS.blit(BGMSurf, (0,0))
            txt = FontObj.render(str(mgr.page), True, C(style.font.color))
            rect = txt.get_rect()
            rect.center=(style.minimum.width//2,style.minimum.height//2)
            DIS.blit(txt, rect)
        else:
            DIS.blit(BGSurf,(0, 0))
            for n in range(mgr.perpage):
                i, j = n//style.maximum.cols, n%style.maximum.cols
                name = mgr.get_name(n)
                txt = FontObj.render(name, True, C(style.font.color))
                rect = txt.get_rect()
                rect.center=((j+0.5)*style.maximum.block.width,
                             (i+0.5)*style.maximum.block.height)
                DIS.blit(txt, rect)
        pygame.display.update()
    draw_text()
    def mini():
        global MINI
        if MINI == False:
            MINI = True
            SetWindowPos(hwnd, win32con.HWND_DESKTOP,\
                         style.minimum.pos.x,style.minimum.pos.y,
                         style.minimum.width,style.minimum.height,
                         win32con.SWP_NOACTIVATE)#win32con.SWP_NOSIZE)
            DIS.blit(BGMSurf, (0,0))
            pygame.display.update()
            pygame.event.get([MOUSEMOTION, MOUSEBUTTONUP])
    def maxi():
        global MINI
        if MINI == True:
            MINI = False
            mini_clock.tick()
            mgr.page = 0
            # recreate the window may be the most efficient one
            SetWindowPos(hwnd, win32con.HWND_DESKTOP,\
                         style.maximum.pos.x,style.maximum.pos.y,
                         style.maximum.width,style.maximum.height,
                         win32con.SWP_SHOWWINDOW)
            draw_text()
    while True:
        for event in pygame.event.get():
            if MINI == False:
                if event.type == MOUSEBUTTONDOWN:
                    if event.button == 1:
                        start_pos = event.pos
                elif event.type == MOUSEBUTTONUP:
                    if event.button == 1:
                        x0, y0 = event.pos
                        x, y = direction.get_mouse_direction(start_pos, event.pos)
                        if y == 1 or y == -1:
                            mgr.pageturn(-y)
                            draw_text()
                        elif x == -1:
                            mini()
                        elif (3 < x0 < style.maximum.width-3 or\
                              3 < y0 < style.maximum.height-3):
                            #to avoid misclicking or launch the shortcut while maximized
                            mgr.launch((y0//style.maximum.block.height)*style.maximum.cols+
                                       x0//style.maximum.block.width)
                            mini()
                    elif event.button in (5, 4):
                        mgr.pageturn(event.button*2-9)
                        draw_text()
                elif event.type == KEYDOWN:
                    if event.key == 276:
                        mini()
                elif event.type == ACTIVEEVENT:
                    # lose focus
                    if event.gain == 0 and event.state == 2:
                        time = mini_clock.tick()
                        print(time)
                        if time > 700:
                            mini()
            elif MINI == True:
                if event.type == MOUSEBUTTONUP:
                    if event.button == 1:
                        if MOVING == False :
                            print('maximizing', event.pos)
                            maxi()
                        else:
                            MOVING = False
                elif event.type == MOUSEMOTION:
                    if event.buttons == (1, 0, 0):
                        MOVING = True
                        x0, y0 = GetCursorPos()
                        SetWindowPos(hwnd, win32con.HWND_DESKTOP,
                                     x0-style.minimum.width//2,
                                     y0-style.minimum.height//2,0,0,
                                     win32con.SWP_NOSIZE)
                        pygame.event.get([MOUSEMOTION, MOUSEBUTTONUP])
                if event.type == KEYDOWN:
                    if event.key == 13:
                        maxi()
            if event.type == QUIT:
                pygame.quit()
                return
            elif event.type == KEYDOWN:
                if event.key == K_ESCAPE:
                    pygame.quit()
                    return
                elif 49 <= event.key <= 57:
                    mgr.launch(event.key-49)
                    mini()
                elif event.key in (280, 281):
                    mgr.pageturn(event.key *2-561)
                    draw_text()
                elif event.key == K_F4 and event.mod != 0:
                    ShowWindow(hwnd, win32con.SW_HIDE)
            elif event.type == MOUSEBUTTONUP:
                if event.button == 3:
                    pygame.quit()
                    return

        pygame.time.wait(50)
if  __name__  == '__main__':
    main()
